__author__ = 'sowmya'

from sls_client.records import *
from sls_client.query import *
import json
import socket



def get_ma_for_host(hostname):

    ipaddresses={}
    if(not hostname):
        raise Exception("Null Parameter", "Hostname parameter is empty.")
    ipaddr = socket.getaddrinfo(hostname, None)

    ipaddresses[hostname]=1

    #ipaddr is a 5-tuple and the ipv4 or ipv6 is the last element in the tuple.
    for ip in ipaddr:
        if(ip[4][0]):
            ipaddresses[ip[4][0]]=1

    response=[]
    result={}
    queryKeys = ['psmetadata-dst-address','psmetadata-src-address']
    for queryKey in queryKeys:
        for ip in ipaddresses:
            queryString = queryKey+"="+ip
            response += query(queryString)


    for record in response:
        malocators = record[u'psmetadata-ma-locator']
        for malocator in malocators:
            malocatorString = malocator.encode("ascii")
            result[malocatorString] = 1

    return result.keys()

def get_ma_for_host_json(hostname):
    hostlist = get_ma_for_host(hostname)
    result={}
    result["ma-hosts"] = []
    result["search-query"] = hostname
    if(hostlist):
        result["ma-hosts"] = hostlist

    json_output = json.dumps(result)
    return json_output


__author__ = 'sowmya'

from sls_client.records import *
from sls_client.query import *
import json
import socket

HOST_NAME='host-name'
HOST_INTERFACE='host-net-interfaces'
INTERFACE_ADDRESS='interface-addresses'
RECORD_URI = 'uri'
INTERFACE_NAME='interface-name'
HOST_ADMINISTRATORS = 'host-administrators'

def get_host_info(hostname):

    if(not hostname):
        raise Exception("Null Parameter", "Hostname parameter is empty.")

    #Try hostname. If it fails, try host name for interface. If that also fails, finally do dns resolution
    queryString = HOST_NAME+"="+hostname
    response = query(queryString)
    hosts=[]
    if response:
        uris=__get_interface_uris_for_host(response)

        interfaces = __get_interfaces(uris)

        #get interfaces
        for interface in interfaces:
            for result in response:
                result[HOST_INTERFACE]=interface
        hosts=response

    else:
        queryString = INTERFACE_ADDRESS+"="+hostname
        response = query(queryString)

        if response:
            hosts = __get_hosts_for_interfaces(response)
        else:
            #DNS resolution
            ipaddr = socket.getaddrinfo(hostname, None)
            ipaddresses={}
            #ipaddr is a 5-tuple and the ipv4 or ipv6 is the last element in the tuple.
            for ip in ipaddr:
                if(ip[4][0]):
                    ipaddresses[ip[4][0]]=1

            ipaddresses[hostname]=1
            hosts=[]
            for ip in ipaddresses:
                queryString = INTERFACE_ADDRESS+"="+ip
                response = query(queryString)

                if response:
                    hosts += __get_hosts_for_interfaces(response)


    for host in hosts:
        admin = __get_admin_info(host)
        host[HOST_ADMINISTRATORS] = admin

    return hosts


def __get_interface_uris_for_host(list_of_host):
    uris={}
    for host in list_of_host:
        for interface in host[HOST_INTERFACE]:
            uris[interface]=1
    return uris

def __get_hosts_for_interfaces(list_of_interfaces):
    hostResponse=[]
    for interface in list_of_interfaces:
        uri = interface[RECORD_URI]
        hostQuery=HOST_INTERFACE+"="+uri
        response = query(hostQuery)
        for host in response:
            host[HOST_INTERFACE] = interface

        hostResponse+=response

    return hostResponse

def __get_admin_info(host):
    adminUris = host[HOST_ADMINISTRATORS]
    response=[]
    if(adminUris):
        for adminUri in adminUris:
            queryString = RECORD_URI+"="+adminUri
            response += query(queryString)
        return response


def __get_interfaces(list_of_uris):
    interfaceResponse = []
    for uri in list_of_uris:
        interfaceQuery= RECORD_URI+'='+uri
        interfaceResponse += query(interfaceQuery)

    return interfaceResponse

def get_host_info_json(hostname):
    hostlist = get_host_info(hostname)
    result={}
    result["host"] = []
    result["search-query"] = hostname
    if(hostlist):
        result["host"] = hostlist

    json_output = json.dumps(result)
    return json_output



from operator import itemgetter
import urllib
from urlparse import parse_qs

import requests

_concurrency_enabled=False
try:
    import concurrent.futures
    _MAX_CONCURRENT_REQUESTS = 4
    _concurrency_enabled = True
except ImportError:
    pass

_LS_HINTS = "http://ps-west.es.net:8096/lookup/activehosts.json"

def get_hosts():
    hosts = requests.get(_LS_HINTS).json().get("hosts", [])
    hosts = sorted(hosts, key=lambda v: v.get("priority", ""), reverse=True)
    for host in hosts:
        if host.get("status", "") != "alive" or not host.get("locator", ""):
            hosts.remove(host)
    return hosts

def refresh_hosts():
    _ls_hosts = get_hosts()

_ls_hosts = get_hosts()

def query(query="", hosts=_ls_hosts):
    try:
        query = hash_to_query(query)
    except:
        pass
    records = []
    urls = [(host["locator"] + "?" + query) for host in hosts]
    if _concurrency_enabled:
        with concurrent.futures.ThreadPoolExecutor(max_workers=_MAX_CONCURRENT_REQUESTS) as pool:
            for response in pool.map(get_url, urls):
                if response is None:
                    continue
                ls_host = response.url.split("lookup/records")[0]
                for record in response.json():
                    record["ls-host"] = ls_host
                    records.append(record)
    else:
        for url in urls:
            response = get_url(url)
            if response is None:
                continue
            ls_host = response.url.split("lookup/records")[0]
            for record in response.json():
                if record.get("uri", "") and record.get("type", [ False ])[0]:
                    record["ls-host"] = ls_host
                    records.append(record)
    return records
    
def hash_to_query(query_hash={}):
    query = urllib.urlencode(sorted(query_hash.items(), key=lambda v: v[0]), True)
    return query

def query_to_hash(query=""):
    query_hash = parse_qs(query)
    return query_hash

def get_url(url):
    try:
        return requests.get(url)
    except:
        pass
    return None



#!/usr/bin/env python

from setuptools import setup
from setuptools import find_packages

setup(
    name="sls-client",
    version="1.0.8",
    description="A python client for the Simple Lookup Service",
    long_description=open("README.rst").read(),
    author=["Andrew Sides","Sowmya Balasubramanian"],
    author_email=["asides@es.net","sowmya@es.net"],
    url="https://github.com/esnet/python-sls-client",
    license="3-clause BSD License",
    package_data={
        "sls-client":["sls-client/LICENSE"]
    },
    packages=['sls_client'],
    scripts=['clients/sls_dig','clients/find_ps_ma'],
    install_requires=[
        "IPy",
        "YURL",
        "isodate",
        "futures",
        "requests",
        "voluptuous"
    ],
    zip_safe=False,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Networking"
    ],
)
